//v-longtap

Vue.directive('longtap', {
    bind: function (el, binding, vNode) {
        if (typeof binding.value !== 'function') {
            const compName = vNode.context.name;
            let warn = `[longtap:] provided expression '${binding.expression}' is not a function, but has to be `;
            if (compName) { warn += `Found in component '${compName}' ` }
            console.warn(warn);
        }

        let pressTimer = null;
        let start = (e) => {
            if (e.type === 'click' && e.button !== 0) {
                return;
            }
            if (pressTimer === null) {
                pressTimer = setTimeout(() => {
                    handler();
                }, 1000)
            }
        }

        let cancel = (e) => {
            if (pressTimer !== null) {
                clearTimeout(pressTimer);
                pressTimer = null;
            }
        }

        const handler = (e) => {
            binding.value(e)
        }

        el.addEventListener("mousedown", start);
        el.addEventListener("touchstart", start);
        el.addEventListener("click", cancel);
        el.addEventListener("mouseout", cancel);
        el.addEventListener("touchend", cancel);
        el.addEventListener("touchcancel", cancel);
    }
})


    // <template>
    //     <div class="double">
    //         <!-- 长按触发incrementPlusTen函数，点击触发incrementPlusOne函数 -->
    //         <button v-longtap="incrementPlusTen" @click="incrementPlusOne">点击我啦</button>

    //     </div>
    // </template>

    // <script>
    // export default {
    //     data() {
    //         return {
    //             value: 10
    //         }
    //     },
    //     methods: {
    //         // 增加1
    //         incrementPlusOne() {
    //             // this.value++
    //             console.log("我是点击事件");

    //         },
    //         // 增加10
    //         incrementPlusTen() {
    //             // this.value += 10
    //             console.log("我是长按事件");
    //         }
    //     }


    // }
    // </script>











    // v-tap
    (function (factory) {
        if (typeof define === 'function' && define.amd) {
            define(function () { return factory; });
        } else if (typeof exports == "object") {
            module.exports = factory;
        } else {
            Vue.use(factory);
        }
    }({
        install: function (Vue) {
            Vue.directive('tap', this.master);
        },
        master: {
            bind: function (el, binding) {
                var isTouch = "ontouchend" in document;

                el.execFun = function (binding) {
                    return function (e) {
                        var data = binding.value;

                        if (!data) {
                            if (el.href && !binding.modifiers.prevent) {
                                return window.location = el.href;

                            } else if (el.tagName == 'INPUT' || el.tagName == 'TEXTAREA') {
                                return el.focus();
                            } else {
                                return;
                            }
                        }
                        var getEl = data.indexOf('$el'),
                            getEvent = data.indexOf('$event');

                        getEl != -1 && (data[getEl] = el);
                        getEvent != -1 && (data[getEvent] = e);

                        data[0].apply(this, data.slice(1));
                    }
                };
                el.exec = el.execFun(binding);
                el.tapInfo = {};

                if (isTouch) {
                    //touchstart
                    el.addEventListener('touchstart', function (e) {
                        var t = e.touches[0];

                        el.tapInfo.startX = t.pageX;
                        el.tapInfo.startY = t.pageY;
                        el.tapInfo.sTime = + new Date;
                        binding.modifiers.stop && (e.stopPropagation());
                        binding.modifiers.prevent && (e.preventDefault());
                    });
                    //touchend
                    el.addEventListener('touchend', function (e) {
                        var t = e.changedTouches[0];

                        el.tapInfo.endX = t.pageX;
                        el.tapInfo.endY = t.pageY;
                        if ((+ new Date) - el.tapInfo.sTime < 300) {
                            if (Math.abs(el.tapInfo.endX - el.tapInfo.startX) + Math.abs(el.tapInfo.endY - el.tapInfo.startY) < 20) {
                                binding.modifiers.stop && (e.stopPropagation());
                                e.preventDefault();
                                el.exec(e);
                            }
                        }
                    });
                } else {
                    //click
                    el.addEventListener('click', function (e) {
                        binding.modifiers.stop && (e.stopPropagation());
                        binding.modifiers.prevent && (e.preventDefault());
                        el.exec(e);
                    });

                }
            },
            componentUpdated: function (el, binding) {
                el.exec = el.execFun(binding);
            },
            unbind: function (el) {
                el.execFun = el.exec = el.tapInfo = null;
            }
        }
    }))



















/**
 * 处理 移动设备上的 点击、长按、左右上下滑动 事件
 * =========================================
 * 导出了7个自定义指令：
 *  v-tap： tap点击事件
 *  v-swipe： swipe滑动事件
 *  v-swipeleft： swipeleft左滑事件
 *  v-swiperight： swiperight右滑事件
 *  v-swipedown： swipedown下滑
 *  v-swipeup： swipeup上滑
 *  v-longtap： longtap长按
 * =========================================
 * 包含四个参数
 * stop 阻止冒泡
 * prevent 阻止默认事件
 * self 只当在 event.target 是当前元素自身时触发处理函数
 * once 执行一次后解绑
 * @example v-tap.stop.prevent.self.once
 * =========================================
 * 事件绑定有两种方式
 * @example
 *  1. v-tap="showDialog" 绑定一个方法对象
 *  2. v-tap="{fn:click123, param1:1, param2:2, param3:{aaa:'123'} ...}"
 *      绑定一个JSON字面量，fn是执行的方法，后边的是需要传递的参数
 * 事件回调参数
 * @param 第一个参数是event，事件对象
 * @param 第二个参数是 binding.value，也就是v-tap=""双引号中的部分（如示例2，第二个参数就是 {fn:click123, param1:1, param2:2, param3:{aaa:'123'} ...}）
 * =========================================
 * @update
 *  1. 根据MUI进行了滑动事件的判断修正
 *  2. 根据TouchEvent修正了点击位置的判断
 *
 */

import Vue from 'vue';

/**
 * vue上点击事件处理类
 */

class VueTouch {
    /**
     * @param el
     * @param binding
     * @param type
     */
    constructor(el, binding, type) {

        let g = this;
        g.obj = el;
        g.binding = binding;
        g.touchType = type;
        g.firstTouchPosition = { x: 0, y: 0 };
        g.firstTouchTime = 0;
        g.callBack = typeof (binding.value) === "object" ? binding.value.fn : binding.value;
        g.moved = false;
        g.leaved = false;
        g.longTouched = false;

        // 事件监听回调集合
        let _listener = Object.create(null);

        // 事件方法
        let _listen = (type) => {
            return function (e) {
                let { stop, prevent, self, once } = g.binding.modifiers;

                // 配置判断
                if (stop) e.stopPropagation();
                if (prevent) e.preventDefault();
                if (once) g.obj.removeEventListener("touch" + type, _listener[type]);
                if (self && e.target !== e.currentTarget) return;

                g[type](e);
            }
        };

        _listener.start = _listen('start');
        _listener.end = _listen('end');
        _listener.move = _listen('move');

        this.b .addEventListener("touchstart", _listener.start, false);
        this.obj.addEventListener("touchend", _listener.end, false);
        this.obj.addEventListener("touchmove", _listener.move, false);
    }

    start(e) {

        let g = this;

        // 初始化点击状态
        g.moved = false;
        g.leaved = false;
        g.longTouched = false;

        g.firstTouchPosition = g.getMultiCenter(e.changedTouches);
        g.firstTouchTime = e.timeStamp;
        g.time = setTimeout(function () {
            if (!g.leaved && !g.moved) {
                g.touchType === "longtap" && g.callBack(e, g.binding.value);
                g.longTouched = true;
            }
        }.bind(g), 1000);
    }

    end(e) {

        let g = this;

        let { x, y } = g.getMultiCenter(e.changedTouches);
        let _disX = x - g.firstTouchPosition.x;
        let _disY = y - g.firstTouchPosition.y;
        let _dis = Math.sqrt(_disX * _disX + _disY * _disY);
        let _timeDis = e.timeStamp - g.firstTouchTime;

        clearTimeout(g.time);

        let _angle = this.getAngle(_disX, _disY);

        if (_dis > 18 && _timeDis < 300) {
            g.touchType === "swipe" && g.callBack(e, g.binding.value);
            if (_angle >= 60 && _angle <= 120)
                g.touchType === "swipedown" && g.callBack(e, g.binding.value);
            if (_angle <= -60 && _angle >= -120)
                g.touchType === "swipeup" && g.callBack(e, g.binding.value);
            if (_angle <= 20 && _angle >= -20)
                g.touchType === "swipeleft" && g.callBack(e, g.binding.value);
            if ((_angle <= -160 && _angle > -180) || (_angle >= 160 && _angle <= 180))
                g.touchType === "swiperight" && g.callBack(e, g.binding.value);
        } else {
            if (!g.longTouched && !g.moved) {
                g.touchType === "tap" && g.callBack(e, g.binding.value);
                g.leaved = true;
            }
        }
    }

    move() {
        this.moved = true;
    }

    /**
     * 获取点击集合的中心坐标
     * @param touches touch对象集合
     * @return {{x: number, y: number}}
     */
    getMultiCenter(touches) {

        let g = this, x = 0, y = 0;

        const _length = touches.length;

        for (let i = 0; i < _length; i++) {
            x += touches[i].pageX;
            y += touches[i].pageY;
        }

        return {
            x: Math.round(x / _length),
            y: Math.round(y / _length)
        };
    };

    getAngle(disX, disY) {
        let g = this;

        if (typeof disX !== 'number' || typeof disY !== 'number')
            return 45;

        return Math.atan2(disY, disX) * 180 / Math.PI;
    }
}


Vue.directive("tap", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "tap");
    }
});
Vue.directive("swipe", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "swipe");
    }
});
Vue.directive("swipeleft", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "swipeleft");
    }
});
Vue.directive("swiperight", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "swiperight");
    }
});
Vue.directive("swipedown", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "swipedown");
    }
});
Vue.directive("swipeup", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "swipeup");
    }
});
Vue.directive("longtap", {
    bind: function (el, binding) {
        new VueTouch(el, binding, "longtap");
    }
})










//v-swiper    可能有问题
touch: {
    bind: function (el, binding, vnode) {
        var touchType = binding.arg; //传入的模式 press swipeRight swipeLeft swipeTop swipeDowm Tap
        var timeOutEvent = 0;
        var direction = '';
        //滑动处理
        var startX, startY;
        //返回角度
        function GetSlideAngle(dx, dy) {
            return Math.atan2(dy, dx) * 180 / Math.PI;
        }
        //根据起点和终点返回方向 1：向上，2：向下，3：向左，4：向右,0：未滑动
        function GetSlideDirection(startX, startY, endX, endY) {
            var dy = startY - endY;
            var dx = endX - startX;
            var result = 0;
            //如果滑动距离太短
            if (Math.abs(dx) < 2 && Math.abs(dy) < 2) {
                return result;
            }
            var angle = GetSlideAngle(dx, dy);
            if (angle >= -45 && angle < 45) {
                result = 'swiperight';
            } else if (angle >= 45 && angle < 135) {
                result = 'swipeup';
            } else if (angle >= -135 && angle < -45) {
                result = 'swipedown';
            }
            else if ((angle >= 135 && angle <= 180) || (angle >= -180 && angle < -135)) {
                result = 'swipeleft';
            }
            return result;
        }
        el.addEventListener('touchstart', function (ev) {
            startX = ev.touches[0].pageX;
            startY = ev.touches[0].pageY;
            //判断长按
            timeOutEvent = setTimeout(() => {
                timeOutEvent = 0;
                if (touchType === 'press') {
                    binding.value()
                }
            }, 500);
        }, false);
        el.addEventListener('touchmove', function (ev) {
            clearTimeout(timeOutEvent)
            timeOutEvent = 0;
        });
        el.addEventListener('touchend', function (ev) {
            var endX, endY;
            endX = ev.changedTouches[0].pageX;
            endY = ev.changedTouches[0].pageY;
            direction = GetSlideDirection(startX, startY, endX, endY);
            clearTimeout(timeOutEvent)
            switch (direction) {
                case 0:
                    break;
                case 'swipeup':
                    if (touchType === 'swipeup') {
                        binding.value()
                    }
                    break;
                case 'swipedown':
                    if (touchType === 'swipedown') {
                        binding.value()
                    }
                    break;
                case 'swipeleft':
                    if (touchType === 'swipeleft') {
                        binding.value()
                    }
                    break;
                case 'swiperight':
                    if (touchType === 'swiperight') {
                        binding.value()
                    }
                    break;
                default:
            }
        }, false);
    }
}

















//v-drag
Vue.directive('drag', {
    bind: function(el, binding) {
        let isDrag = true;
        let tempX = 0;
        let x = 0;
        let tempY = 0;
        let y = 0;

        el.ontouchstart = e => {
            isDrag = true;
            tempX = parseInt(el.style.left + 0);
            tempY = parseInt(el.style.top + 0);
            x = e.touches[0].pageX;
            y = e.touches[0].pageY;
        };

        el.ontouchmove = e => {
            if (isDrag) {
                let curX = tempX + e.touches[0].pageX - x;
                let curY = tempY + e.touches[0].pageY - y;
                let height = document.documentElement.clientHeight;
                if (binding.value) {
                    height = el.parentElement.clientHeight;
                }
                let clientWidth = document.documentElement.clientWidth - el.clientWidth - 2;
                let clientHeight = height - el.clientHeight - 2;
                //边界判断
                curX = curX < 0 ? 0 : curX;
                curY = curY < 0 ? 0 : curY;
                curX = curX < clientWidth ? curX : clientWidth;
                curY = curY < clientHeight ? curY : clientHeight;

                el.style.left = curX + "px";
                el.style.top = curY + "px";
                //阻止浏览器继续处理触摸(和鼠标)事件。
                e.preventDefault();
            }
        };
        el.ontouchend = e => {
            isDrag = false;
        };
    }
});








/**
 * 该指令用于设定自动聚焦,使用时,需要聚焦的元素必须设置data-index属性,
 * 用法 父元素 v-auto-focus="focusCtrl" :data-action="actionType" :data-current="focusIndex"
 * 需要聚焦的元素需要设置索引 data-index="index"
 */
export default {

    inserted: function (el) {
        const allFocusEls = getAllFocusEls(el)  // 获取需要聚焦的input元素组
        let current = getTargetIndex(el, allFocusEls)
        if (!current || current < 0 || current >= allFocusEls.length) {  // 如果没有设置data-current,或者current的数值范围不符合要求,则默认聚焦到第一个输入框
            current = 0
        }
        const currentEl = allFocusEls[current]

        autoFocus(currentEl)
    },
    update: function (el, { value, oldValue }) {
        if (value !== oldValue) {
            focusCtrl(el)
        }
    },
}

const autoFocus = function (focusEl) {
    let elInput = focusEl
    if (['INPUT', 'TEXTAREA'].indexOf(focusEl.tagName.toUpperCase()) === -1) {
        elInput = focusEl.querySelector('input,textarea')
    }
    elInput && elInput.focus()
}
const getAllFocusEls = function (el) {
    return el.querySelectorAll('[data-index]')
}

/**
 * 聚焦行为控制
 * next 聚焦到下一个元素
 * prev 聚焦到上一个元素
 * first 聚焦到第一个元素
 * last 聚焦到最后一个元素
 * jump 跳转到指定的元素
 * @param el
 */
const focusCtrl = function (el) {
    const action = el.dataset.action
    const allFocusEls = getAllFocusEls(el)
    const focusLen = allFocusEls.length
    let current = getTargetIndex(el, allFocusEls)
    switch (action) {
        case 'next':  // 如果action为next,则聚焦到下一个输入框
            if (current >= focusLen - 1) {
                current = focusLen - 1
            } else {
                current++
            }
            autoFocus(allFocusEls[current])
            break
        case 'prev':  // 如果action为prev,则聚焦到上一个输入框
            if (current <= 0) {
                current = 0
            } else {
                current--
            }
            autoFocus(allFocusEls[current])
            break
        case 'first': // 如果为first,则聚焦到第一个输入框
            current = 0
            autoFocus(allFocusEls[current])
            break;
        case 'last': // 如果为last,则聚焦到最后一个输入框
            current = focusLen - 1
            autoFocus(allFocusEls[current])
            break
        case 'jump': // 如果为jump,则获取focusIndex,跳转到对应的输入框
            if (current >= 0 && current < focusLen) {
                autoFocus(allFocusEls[current])
            }
            break
    }
}

const getTargetIndex = function (el, collection) {
    const target = document.querySelector(`[data-index="${el.dataset.current}"]`)
    return Array.from(collection).indexOf(target)
}

const install = function (Vue) {
    if (install.installed) return;
    Vue.directive('auto-focus', autoFocus)
}
if (typeof window !== 'undefined' && window.Vue) {
    install(window.Vue);
};

export default {
    install,
}













//吸顶

import Vue from 'vue'
import VueStickto from 'vue-stickto'

Vue.use(VueStickto)

import {stick} from 'stickto/index.js'

export default {
    install: function (Vue, options) {
        Vue.directive('stickto', {
            inserted (el) {
                stick(el)
            },
        })
    },
}










//img error
Vue.directive('imgerror', async function (el, binding) {
    let imgURL = binding.value;//获取图片地址
    let defaultURL = el.getAttribute('default-img');//获取默认图片地址
    if (imgURL) {
        let exist = await imageIsExist(imgURL);
        if (exist) {
            el.setAttribute('src', imgURL);
        } else {
            if (defaultURL) {
                el.setAttribute('src', defaultURL);
            }
        }
    }
})
const imageIsExist = function (url) {
    return new Promise((resolve) => {
        var img = new Image();
        img.onload = function () {
            if (this.complete == true){ 
                resolve(true);
                img = null;
            }
        }
        img.onerror = function () {
            resolve(false);
            img = null;
        }
        img.src = url;
    })
}